@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008-2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Connector;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.29';
+our $VERSION = '0.31';
 
 use POSIX qw( EINPROGRESS );
 use Socket qw( SOL_SOCKET SO_ERROR );
@@ -70,6 +70,10 @@ To be aware of individual failures, the optional C<on_fail> callback can be
 used. This will be invoked on each individual C<socket()> or C<connect()>
 failure, which may be useful for debugging or logging.
 
+Because this module simply uses the C<getaddrinfo> resolver, it will be fully
+IPv6-aware if the underlying platform's resolver is. This allows programs to
+be fully IPv6-capable.
+
 =cut
 
 # Internal constructor
@@ -223,7 +227,9 @@ an array, with at least the following elements:
 
 The first three arguments will be passed to a C<socket()> call and, if
 successful, the fourth to a C<connect()> call on the resulting socket. Any
-trailing elements will be ignored.
+trailing elements will be ignored. Note that C<$address> must be a packed
+socket address, such as returned by C<pack_sockaddr_in> or
+C<pack_sockaddr_un>. See also the C<EXAMPLES> section,
 
 =item on_connected => CODE
 
@@ -231,6 +237,24 @@ A continuation that is invoked on a successful C<connect()> call to a valid
 socket. It will be passed the connected socket handle, as an C<IO::Socket>
 object.
 
+ $on_connected->( $handle )
+
+=item on_stream => CODE
+
+An alternative to C<on_connected>, a continuation that is passed an instance
+of L<IO::Async::Stream> when the socket is connected. This is provided as a
+convenience for the common case that a Stream object is required as the
+transport for a Protocol object.
+
+ $on_stream->( $stream )
+
+=item on_socket => CODE
+
+Similar to C<on_stream>, but constructs an instance of L<IO::Async::Socket>.
+This is most useful for C<SOCK_DGRAM> or C<SOCK_RAW> sockets.
+
+ $on_socket->( $socket )
+
 =item on_connect_error => CODE
 
 A continuation that is invoked after all of the addresses have been tried, and
@@ -302,11 +326,30 @@ sub connect
    my $self = shift;
    my ( %params ) = @_;
 
-   my $timeout = $params{timeout};
-   $timeout = 30 if !defined $timeout;
-
    # Callbacks
-   my $on_connected     = $params{on_connected}     or croak "Expected 'on_connected' callback";
+   my $on_connected;
+   if( $on_connected = delete $params{on_connected} ) {
+      # all fine
+   }
+   elsif( my $on_stream = delete $params{on_stream} ) {
+      # TODO: It doesn't make sense to put a SOCK_DGRAM in an
+      # IO::Async::Stream but currently we don't detect this
+      $on_connected = sub {
+         my ( $handle ) = @_;
+         $on_stream->( IO::Async::Stream->new( handle => $handle ) );
+      };
+   }
+   elsif( my $on_socket = delete $params{on_socket} ) {
+      require IO::Async::Socket;
+      $on_connected = sub {
+         my ( $handle ) = @_;
+         $on_socket->( IO::Async::Socket->new( handle => $handle ) );
+      };
+   }
+   else {
+      croak "Expected 'on_connected' or 'on_stream' callback";
+   }
+
    my $on_connect_error = $params{on_connect_error} or croak "Expected 'on_connect_error' callback";
 
    my $on_fail = $params{on_fail};
@@ -320,16 +363,7 @@ sub connect
       my $loop = $self->{loop};
 
       my $family   = $params{family}   || 0;
-
-      my $socktype;
-      if( $params{type} ) {
-         carp( "'type' deprecated, use 'socktype' instead" );
-         $socktype = $params{type} || 0;
-      }
-      else {
-         $socktype = $params{socktype} || 0;
-      }
-
+      my $socktype = $params{socktype} || 0;
       my $protocol = $params{protocol} || 0;
       my $flags    = $params{flags}    || 0;
 
@@ -359,6 +393,44 @@ sub connect
 
 __END__
 
+=head1 EXAMPLES
+
+=head2 Passing Packed Socket Addresses
+
+The C<addr> or C<addrs> parameters should contain a packed socket address.
+This example shows how to use the C<Socket> functions to construct one for
+TCP port 8001 on address 10.0.0.1:
+
+ use Socket qw( PF_INET SOCK_STREAM pack_sockaddr_in inet_aton );
+
+ ...
+
+ $loop->connect(
+    addr => [
+       PF_INET,
+       SOCK_STREAM,
+       0, # Don't need to supply a protocol as kernel will do that
+       pack_sockaddr_in( 8001, inet_aton( "10.0.0.1" ) ),
+    ],
+    ...
+ );
+
+This example shows another way to connect to a UNIX socket at F<echo.sock>.
+
+ use Socket qw( PF_UNIX SOCK_STREAM pack_sockaddr_un );
+
+ ...
+
+ $loop->connect(
+    addr => [
+       PF_UNIX,
+       SOCK_STREAM,
+       0, # Don't need to supply a protocol as kernel will do that
+       pack_sockaddr_un( "echo.sock" ),
+    ],
+    ...
+ );
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
