@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.30';
+our $VERSION = '0.31';
 
 use IO::Async::Handle;
 
@@ -27,25 +27,24 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
 =head1 SYNOPSIS
 
  use IO::Async::Listener;
- use IO::Async::Stream;
 
  use IO::Async::Loop;
  my $loop = IO::Async::Loop->new();
 
  my $listener = IO::Async::Listener->new(
-    on_accept => sub {
-       my ( $newclient ) = @_;
-
-       $loop->add( IO::Async::Stream->new(
-          handle => $newclient,
+    on_stream => sub {
+       my ( undef, $stream ) = @_;
 
+       $stream->configure(
           on_read => sub {
              my ( $self, $buffref, $closed ) = @_;
              $self->write( $$buffref );
              $$buffref = "";
              return 0;
           },
-       ) );
+       );
+       
+       $loop->add( $stream );
     },
  );
 
@@ -128,14 +127,14 @@ of L<IO::Async::Stream> when a new client connects. This is provided as a
 convenience for the common case that a Stream object is required as the
 transport for a Protocol object.
 
- $on_stream->( $stream )
+ $on_stream->( $self, $stream )
 
 =item on_socket => CODE
 
 Similar to C<on_stream>, but constructs an instance of L<IO::Async::Socket>.
 This is most useful for C<SOCK_DGRAM> or C<SOCK_RAW> sockets.
 
- $on_socket->( $socket )
+ $on_socket->( $self, $socket )
 
 =item handle => IO
 
@@ -155,6 +154,7 @@ sub configure
    }
    elsif( exists $params{on_stream} ) {
       my $on_stream = delete $params{on_stream};
+      require IO::Async::Stream;
       # TODO: It doesn't make sense to put a SOCK_DGRAM in an
       # IO::Async::Stream but currently we don't detect this
       $self->{on_accept} = sub {
@@ -278,7 +278,9 @@ an array, with at least the following elements:
 The first three arguments will be passed to a C<socket()> call and, if
 successful, the fourth to a C<bind()> call on the resulting socket. The socket
 will then be C<listen()>ed to put it into listening mode. Any trailing
-elements in this array will be ignored.
+elements in this array will be ignored. Note that C<$address> must be a packed
+socket address, such as returned by C<pack_sockaddr_in> or
+C<pack_sockaddr_un>. See also the C<EXAMPLES> section,
 
 =back
 
@@ -364,6 +366,9 @@ be set. To prevent this, pass a false value such as 0.
 
 =back
 
+As a convenience, it also supports a C<handle> argument, which is passed
+directly to C<configure>.
+
 =cut
 
 sub listen
@@ -374,6 +379,12 @@ sub listen
    my $loop = $self->get_loop;
    defined $loop or croak "Cannot listen when not a member of a Loop"; # TODO: defer?
 
+   if( exists $params{handle} ) {
+      my $handle = $params{handle};
+      $self->configure( handle => $handle );
+      return;
+   }
+
    # Shortcut
    if( $params{addr} and not $params{addrs} ) {
       $params{addrs} = [ delete $params{addr} ];
@@ -472,6 +483,87 @@ sub listen
 
 __END__
 
+=head1 EXAMPLES
+
+=head2 Listening on UNIX Sockets
+
+The C<handle> argument can be passed an existing socket already in listening
+mode, making it possible to listen on other types of socket such as UNIX
+sockets.
+
+ use IO::Async::Listener;
+ use IO::Socket::UNIX;
+
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
+
+ my $listener = IO::Async::Listener->new(
+    on_stream => sub {
+       my ( undef, $stream ) = @_;
+
+       $stream->configure(
+          on_read => sub {
+             my ( $self, $buffref, $closed ) = @_;
+             $self->write( $$buffref );
+             $$buffref = "";
+             return 0;
+          },
+       );
+       
+       $loop->add( $stream );
+    },
+ );
+
+ $loop->add( $listener );
+
+ my $socket = IO::Socket::UNIX->new(
+    Local => "echo.sock",
+    Listen => 1,
+ ) or die "Cannot make UNIX socket - $!\n";
+
+ $listener->listen(
+    handle => $socket,
+ );
+
+ $loop->loop_forever;
+
+=head2 Passing Packed Socket Addresses
+
+The C<addr> or C<addrs> parameters should contain a packed socket address.
+This example shows how to use the C<Socket> functions to construct one for
+TCP port 8001 on address 10.0.0.1:
+
+ use Socket qw( PF_INET SOCK_STREAM pack_sockaddr_in inet_aton );
+
+ ...
+
+ $listener->listen(
+    addr => [
+       PF_INET,
+       SOCK_STREAM,
+       0, # Don't need to supply a protocol as kernel will do that
+       pack_sockaddr_in( 8001, inet_aton( "10.0.0.1" ) ),
+    ],
+    ...
+ );
+
+This example shows another way to listen on a UNIX socket, similar to the
+earlier example:
+
+ use Socket qw( PF_UNIX SOCK_STREAM pack_sockaddr_un );
+
+ ...
+
+ $listener->listen(
+    addr => [
+       PF_UNIX,
+       SOCK_STREAM,
+       0, # Don't need to supply a protocol as kernel will do that
+       pack_sockaddr_un( "echo.sock" ),
+    ],
+    ...
+ );
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
