@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Notifier;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.28';
+our $VERSION = '0.31';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -32,8 +32,7 @@ Usually not directly used by a program, but one valid use case may be:
  my $notifier = IO::Async::Notifier->new();
 
  $notifier->add_child(
-    IO::Async::Stream->new(
-       read_handle => \*STDIN,
+    IO::Async::Stream->new_for_stdin(
        on_read => sub {
           my $self = shift;
           my ( $buffref, $closed ) = @_;
@@ -77,7 +76,13 @@ L<IO::Async::Handle> - event callbacks for a non-blocking file descriptor
 
 =item *
 
-L<IO::Async::Stream> - read and write buffers around an IO handle
+L<IO::Async::Stream> - event callbacks and write bufering for a stream
+filehandle
+
+=item *
+
+L<IO::Async::Socket> - event callbacks and send buffering for a socket
+filehandle
 
 =item *
 
@@ -91,6 +96,10 @@ L<IO::Async::Timer> - base class for Notifiers that use timed delays
 
 L<IO::Async::Signal> - event callback on receipt of a POSIX signal
 
+=item *
+
+L<IO::Async::PID> - event callback on exit of a child process
+
 =back
 
 For more detail, see the SYNOPSIS section in one of the above.
@@ -125,16 +134,9 @@ This function returns a new instance of a C<IO::Async::Notifier> object with
 the given initial values of the named parameters.
 
 Up until C<IO::Async> version 0.19, this module used to implement the IO
-handle features now found in the C<IO::Async::Handle> subclass. To allow for a
-smooth upgrade of existing code, this constructor check for any C<%params> key
-which looks like it belongs there instead. These keys are C<handle>,
-C<read_handle>, C<write_handle>, C<on_read_ready> and C<on_write_ready>. If
-any of these keys are present, then a C<IO::Async::Handle> is returned.
-
-Do not rely on this feature in new code.  This logic exists purely to provide
-an upgrade path from older code that still expects C<IO::Async::Notifier> to
-provide filehandle operations. This will eventually produce a deprecation
-warning at some point in the future, and removed at some point beyond that.
+handle features now found in the C<IO::Async::Handle> subclass. Code that
+needs to use any of C<handle>, C<read_handle>, C<write_handle>,
+C<on_read_ready> or C<on_write_ready> should use L<IO::Async::Handle> instead.
 
 =cut
 
@@ -143,14 +145,12 @@ sub new
    my $class = shift;
    my %params = @_;
 
-   if( $class eq __PACKAGE__ ) {
-      # TODO: This is temporary. Eventually, throw a deprecation warning.
-      foreach my $key ( keys %params ) {
-         if( grep { $key eq $_ } qw( handle read_handle write_handle on_read_ready on_write_ready ) ) {
-            require IO::Async::Handle;
-            return IO::Async::Handle->new( %params );
-         }
-      }
+   if( $class eq __PACKAGE__ and 
+      grep { exists $params{$_} } qw( handle read_handle write_handle on_read_ready on_write_ready ) ) {
+      carp "IO::Async::Notifier no longer wraps a filehandle; see instead IO::Async::Handle";
+
+      require IO::Async::Handle;
+      return IO::Async::Handle->new( %params );
    }
 
    my $self = bless {
@@ -380,6 +380,56 @@ sub _remove_from_loop
    # empty default
 }
 
+=head1 UTILITY METHODS
+
+=cut
+
+=head2 $mref = $notifier->_capture_weakself( $code )
+
+Returns a new CODE ref which, when invoked, will invoke the originally-passed
+ref, with additionally a reference to the Notifier as its first argument. The
+Notifier reference is stored weakly in C<$mref>, so this CODE ref may be
+stored in the Notifier itself without creating a cycle.
+
+For example,
+
+ my $mref = $notifier->_capture_weakself( sub {
+    my ( $notifier, $arg ) = @_;
+    print "Notifier $notifier got argument $arg\n";
+ } );
+
+ $mref->( 123 );
+
+This is provided as a utility for Notifier subclasses to use to build a
+callback CODEref to pass to a Loop method, but which may also want to store
+the CODE ref internally for efficiency.
+
+The C<$code> argument may also be a plain string, which will be used as a
+method name; the returned CODE ref will then invoke that method on the object.
+
+=cut
+
+sub _capture_weakself
+{
+   my $self = shift;
+   my ( $code ) = @_;   # actually bare method names work too
+
+   if( !ref $code ) {
+      my $class = ref $self;
+      my $coderef = $self->can( $code ) or
+         croak qq(Can't locate object method "$code" via package "$class");
+
+      $code = $coderef;
+   }
+
+   weaken $self;
+
+   return sub {
+      unshift @_, $self;
+      goto &$code;
+   };
+}
+
 # Keep perl happy; keep Britain tidy
 1;
 
