@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::PID;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.60_004';
+our $VERSION = '0.31';
 
 use Carp;
 
@@ -23,9 +23,9 @@ C<IO::Async::PID> - event callback on exit of a child process
  use POSIX qw( WEXITSTATUS );
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new;
+ my $loop = IO::Async::Loop->new();
 
- my $kid = $loop->fork(
+ my $kid = $loop->detach_child(
     code => sub {
        print "Child sleeping..\n";
        sleep 10;
@@ -48,27 +48,26 @@ C<IO::Async::PID> - event callback on exit of a child process
 
  $loop->add( $pid );
 
- $loop->run;
+ $loop->loop_forever;
 
 =head1 DESCRIPTION
 
 This subclass of L<IO::Async::Notifier> invokes its callback when a process
 exits.
 
-For most use cases, a L<IO::Async::Process> object provides more control of
-setting up the process, connecting filehandles to it, sending data to and
-receiving data from it.
+This object may be used in one of two ways; as an instance with CODE
+references as callbacks, or as a base class with overridden methods.
 
-=cut
+=over 4
 
-=head1 EVENTS
+=item Subclassing
 
-The following events are invoked, either using subclass methods or CODE
-references in parameters:
+If a subclass is built, then it can override the following methods to handle
+events:
 
-=head2 on_exit $exitcode
+ $self->on_exit( $exitcode )
 
-Invoked when the watched process exits.
+=back
 
 =cut
 
@@ -80,12 +79,14 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item pid => INT
 
-The process ID to watch. Must be given before the object has been added to the
-containing C<IO::Async::Loop> object.
+The process ID to watch. Can only be given at construction time.
 
 =item on_exit => CODE
 
-CODE reference for the C<on_exit> event.
+CODE reference to callback to invoke when the process exits. If not supplied,
+the subclass method will be called instead.
+
+ $on_exit->( $self, $exitcode )
 
 =back
 
@@ -94,22 +95,30 @@ object is removed from the containing C<IO::Async::Loop> object.
 
 =cut
 
+sub _init
+{
+   my $self = shift;
+   my ( $params ) = @_;
+
+   # Not valid to watch for 0
+   my $pid = delete $params->{pid} or croak "Expected 'pid'";
+
+   $self->{pid} = $pid;
+
+   $self->SUPER::_init( $params );
+}
+
 sub configure
 {
    my $self = shift;
    my %params = @_;
 
-   if( exists $params{pid} ) {
-      $self->loop and croak "Cannot configure 'pid' after adding to Loop";
-      $self->{pid} = delete $params{pid};
-   }
-
    if( exists $params{on_exit} ) {
       $self->{on_exit} = delete $params{on_exit};
 
       undef $self->{cb};
 
-      if( my $loop = $self->loop ) {
+      if( my $loop = $self->get_loop ) {
          $self->_remove_from_loop( $loop );
          $self->_add_to_loop( $loop );
       }
@@ -123,21 +132,16 @@ sub _add_to_loop
    my $self = shift;
    my ( $loop ) = @_;
 
-   $self->pid or croak "Require a 'pid' in $self";
-
-   $self->SUPER::_add_to_loop( @_ );
-
    # on_exit continuation gets passed PID value; need to replace that with
    # $self
-   $self->{cb} ||= $self->_replace_weakself( sub {
-      my $self = shift or return;
-      my ( $exitcode ) = @_;
+   $self->{cb} ||= $self->_capture_weakself( sub {
+      my ( $self, $pid, $exitcode ) = @_;
 
-      $self->invoke_event( on_exit => $exitcode );
+      $self->{on_exit} ? $self->{on_exit}->( $self, $exitcode )
+                       : $self->on_exit( $exitcode );
 
-      # Since this is a oneshot, we'll have to remove it from the loop or
-      # parent Notifier
-      $self->remove_from_parent;
+      # Since this is a oneshot, we'll have to remove it from the loop
+      $self->get_loop->remove( $self );
    } );
 
    $loop->watch_child( $self->pid, $self->{cb} );
@@ -151,16 +155,6 @@ sub _remove_from_loop
    $loop->unwatch_child( $self->pid );
 }
 
-sub notifier_name
-{
-   my $self = shift;
-   if( length( my $name = $self->SUPER::notifier_name ) ) {
-      return $name;
-   }
-
-   return $self->{pid};
-}
-
 =head1 METHODS
 
 =cut
@@ -191,10 +185,11 @@ sub kill
    kill $signal, $self->pid or croak "Cannot kill() - $!";
 }
 
+# Keep perl happy; keep Britain tidy
+1;
+
+__END__
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
-
-=cut
-
-0x55AA;
