@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Protocol;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.60_004';
+our $VERSION = '0.31';
 
 use base qw( IO::Async::Notifier );
 
@@ -38,17 +38,6 @@ subclasses
 
 =cut
 
-=head1 EVENTS
-
-The following events are invoked, either using subclass methods or CODE
-references in parameters:
-
-=head2 on_closed
-
-Optional. Invoked when the transport handle becomes closed.
-
-=cut
-
 =head1 PARAMETERS
 
 The following named parameters may be passed to C<new> or C<configure>:
@@ -61,7 +50,9 @@ The C<IO::Async::Handle> to delegate communications to.
 
 =item on_closed => CODE
 
-CODE reference for the C<on_closed> event.
+Optional. A CODE reference to invoke when the transport handle becomes closed.
+
+ $on_closed->( $self )
 
 =back
 
@@ -118,79 +109,6 @@ sub transport
    return $self->{transport};
 }
 
-=head2 $protocol->connect( %args )
-
-Sets up a connection to a peer, and configures the underlying C<transport> for
-the Protocol.
-
-Takes the following named arguments:
-
-=over 8
-
-=item socktype => STRING or INT
-
-Required. Identifies the socket type, and the type of continuation that will
-be used. If this value is C<"stream"> or C<SOCK_STREAM> then C<on_stream>
-continuation will be used; otherwise C<on_socket> will be used.
-
-=item on_connected => CODE
-
-Optional. If supplied, will be invoked once the connection has been
-established.
-
- $on_connected->( $protocol )
-
-=item transport => IO::Async::Handle
-
-Optional. If this is provided, it will immediately be configured as the
-transport (by calling C<configure>), and the C<on_connected> callback will be
-invoked. This is provided as a convenient shortcut.
-
-=back
-
-Other arguments will be passed to the underlying C<IO::Async::Loop> C<connect>
-call.
-
-=cut
-
-sub connect
-{
-   my $self = shift;
-   my %args = @_;
-
-   my $on_connected = delete $args{on_connected};
-
-   if( my $transport = $args{transport} ) {
-      $self->configure( transport => $transport );
-
-      $on_connected->( $self ) if $on_connected;
-
-      return;
-   }
-
-   my $socktype = $args{socktype} or croak "Expected socktype";
-
-   my $on_transport = do {
-      no warnings 'numeric';
-      $socktype eq "stream" || $socktype == Socket::SOCK_STREAM()
-   } ? "on_stream" : "on_socket";
-
-   my $loop = $self->loop or croak "Cannot ->connect a ".ref($self)." that is not in a Loop";
-
-   $loop->connect(
-      %args,
-      socktype => "stream",
-
-      $on_transport => sub {
-         my ( $transport ) = @_;
-
-         $self->configure( transport => $transport );
-
-         $on_connected->( $self ) if $on_connected;
-      },
-   );
-}
-
 =head1 TRANSPORT DELEGATION
 
 The following methods are delegated to the transport object
@@ -227,12 +145,13 @@ sub setup_transport
 
    $transport->configure( 
       on_closed => $self->_capture_weakself( sub {
-         my $self = shift or return;
+         my $self = shift;
          my ( $transport ) = @_;
 
-         $self->maybe_invoke_event( on_closed => );
+         my $on_closed = $self->{on_closed} ||
+                          $self->can( 'on_closed' );
 
-         $self->configure( transport => undef );
+         $on_closed->( $self ) if $on_closed;
       } ),
    );
 }
@@ -254,10 +173,7 @@ sub teardown_transport
    );
 }
 
-=head1 AUTHOR
+# Keep perl happy; keep Britain tidy
+1;
 
-Paul Evans <leonerd@leonerd.org.uk>
-
-=cut
-
-0x55AA;
+__END__
