@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Timer::Countdown;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.28';
+our $VERSION = '0.31';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -42,31 +42,24 @@ C<IO::Async::Timer::Countdown> - event callback after a fixed delay
 
 =head1 DESCRIPTION
 
-This module provides a subclass of L<IO::Async::Timer> for implementing
-one-shot fixed delays. The object implements a countdown timer, which invokes
-its callback after the given period from when it was started. After it has
-expired the Timer may be started again, when it will wait the same period then
-invoke the callback again. A timer that is currently running may be stopped or
-reset.
+This subclass of L<IO::Async::Timer> implements one-shot fixed delays.
+The object implements a countdown timer, which invokes its callback after the
+given period from when it was started. After it has expired the Timer may be
+started again, when it will wait the same period then invoke the callback
+again. A timer that is currently running may be stopped or reset.
 
 For a C<Timer> object that repeatedly runs a callback at regular intervals,
 see instead L<IO::Async::Timer::Periodic>.
 
-This object may be used in one of two ways; with a callback function, or as a
-base class.
+This object may be used in one of two ways; as an instance with CODE
+references as callbacks, or as a base class with overridden methods.
 
 =over 4
 
-=item Callbacks
+=item Subclassing
 
-If the C<on_expire> key is supplied to the constructor, it should contain a
-CODE reference to a callback function to be invoked at the appropriate time:
-
- $on_expire->( $self )
-
-=item Base Class
-
-If a subclass is built, then it can override the C<on_expire> method.
+If a subclass is built, then it can override the following methods to handle
+events:
 
  $self->on_expire()
 
@@ -85,6 +78,8 @@ The following named parameters may be passed to C<new> or C<configure>:
 CODE reference to callback to invoke when the timer expires. If not supplied,
 the subclass method will be called instead.
 
+ $on_expire->( $self )
+
 =item delay => NUM
 
 The delay in seconds after starting the timer until it expires. Cannot be
@@ -104,7 +99,7 @@ sub configure
 
    if( exists $params{on_expire} ) {
       my $on_expire = delete $params{on_expire};
-      ref $on_expire eq "CODE" or croak "Expected 'on_expire' as a CODE reference";
+      ref $on_expire or croak "Expected 'on_expire' as a reference";
 
       $self->{on_expire} = $on_expire;
       undef $self->{cb}; # Will be lazily constructed when needed
@@ -202,13 +197,14 @@ For example, to expire an accepted connection after 30 seconds of inactivity:
 
        on_expire => sub { $stream->close },
     );
+    $stream->add_child( $watchdog );
 
-    my $stream = IO::Async::Stream->new(
+    $stream = IO::Async::Stream->new(
        handle => $newclient,
 
        on_read => sub {
           my ( $self, $buffref, $closed ) = @_;
-          $stream->reset;
+          $watchdog->reset;
 
           ...
        },
@@ -220,7 +216,6 @@ For example, to expire an accepted connection after 30 seconds of inactivity:
 
     $watchdog->start;
 
-    $loop->add( $stream );
     $loop->add( $watchdog );
  }
 
